// ***********************************************************************
// VISUAL MARRIAGE Applet
//
// Visualizes the algorithm for finding a man-optimal stable marriage.
// The simulated algorithm is the one of Gale and Shapley.
//
// by Rolf Bruderer, Oct 2003
// ***********************************************************************

import java.util.*;
import java.awt.*;

public class ColorFactory
	// Class for creating diffrent (non-black) colors.
{

	private Vector colors;
	// all already produced colors (used as seeds later).
	
	private int seed_count;
	// number of available seed colors.
	
	private float distance;
	// distance between new generated colors and already existing colors.
	
	private int direction_index;
	// index of one of the seven possible directions for new colors.
	
	private Vector directions;
	// Vector with all seven possible directions.
	
	private int seed_index;
	// position of seed color for producing new colors.
	
	public ColorFactory(int count)
	// initialize a color factory to produce
	// the given amount 'count' of diffrent colors.
	{
		hueStep = 1.0f / (float)count;
		nextHue = 0.0f;
		satStep = 0.0f;
		nextSat = 1.0f;
		
		/*if (count > 10) {						
			satStep = 0.7f / ((float)count / 10.0f);
			hueStep = 1.0f / 10.0f;
		}*/
	}
	
	private float hueStep;
	// hue diffrence between two following colors 
	// for the produced number of colors.

	private float nextHue = 0.0f;
	// color hue of the next color.
	
	private float nextSat = 1.0f;
	// color value of the next color.
	
	private float satStep = 0.0f;
	// step for value.
	
	private int countPass;
	// count the numbers of time the hue passes 1.0;

	public Color getNextColor() 
	// Gets the next color, that has been produced by this color factory.
	// If the method is called to much times, the smae colors may repeat.
	// If produceColors was not called before, the colors may be to resembling.
	{
		float hue = nextHue;
		float sat = nextSat;
		nextSat = nextSat - satStep;
		if ((nextSat < 0.3f) || (satStep == 0.0f)) {
			nextSat = 1.0f;
			nextHue = nextHue + hueStep;					
		}
		if (nextHue >= 1.0) {
			nextHue = nextHue - 1.0f;
			nextSat = 1.0f;
			countPass++;
		}		
		return createHSVColor(hue, sat, 1.0f - (countPass % 5) * 0.1f);
	}
	
	public Color createHSVColor(float h, float s, float v) 
	// creates a color in the HSV-Color-Space by converting 'h', 's', 'v' to rgb.
	{

		float r = 0.0f, g = 0.0f, b = 0.0f;
		
		if (s == 0.0) {
			r = v;
			g = v;
			b = v;
		}
		else {		

		    // Get h into interval [0 ,6).
			while (h >= 1.0f) {
				h = h - 1.0f;
			}
			while (h < 0.0f) {
				h = h + 1.0f;
			}			
     		h = h * 6.0f;   
			
			
			// get largest integer less than h and fractional part of h.
    		int i = (int)h;
    		float f = h - i;   

			// calculate p, q and t.
		    float p = v * (1.0f - s);
    		float q = v * (1.0f - (s * f));
    		float t = v * (1.0f - (s * (1.0f - f)));

			switch (i) {
				case 0:
        			r = v;
        			g = t;
        			b = p;
					break;
    			case 1:
        		  	r = q;
					g = v;
					b = p;
					break;
    			case 2:
        		  	r = p;
        		  	g = v;
        		  	b = t;
					break;
    			case 3:
        		  	r = p;
        		  	g = q;
        		  	b = v;
					break;
    			case 4:
        		  	r = t;
        		  	g = p;
        		  	b = v;
					break;
    			case 5:
        		  	r = v;
        		  	g = p;
        		  	b = q;
				  	break;
				default:
					System.out.println("ERROR!!!!!!");				
    		}			
			
		}
		
		Color c = new Color(r, g, b);
		return c;	
		
	}	
	
	/* obsolete
	
	public Color createColor()
	// creates a new (not yet used) color.
	{
		
		Color result = null;
		
		// repeat until non-black color found:
		while (result == null) {

			// go from actual color c (accordingly to seed_index)
			// one step in actual direction d (according to direction_index);
			float[] c = (float[]) colors.elementAt (seed_index);
			float[] d = (float[]) directions.elementAt (direction_index);
			float[] r = new float[3];
			r[0] = c[0] + (d[0] * distance);
			r[1] = c[1] + (d[1] * distance);
			r[2] = c[2] + (d[2] * distance);
			
			// check if non-black color has been found
			// and that color is inside color space.
			if (
				((r[0] > 0.5f) || (r[1] > 0.5f) || (r[2] > 0.5f)) 
				&& (r[0] <= 1.0f) && (r[1] <= 1.0f) && (r[2] <= 1.0f)
			) {
				// remember color as seed and as result.
				colors.add (r);
				result = new Color (r[0], r[1], r[2]);				
			}			
						
			// go to next seed color.
			seed_index = seed_index + 1;
	
			// was this the last seed color?
			if (seed_index >= seed_count) {
				// restart with first seed color.
				seed_index = 0;				
				// was this the last direction?
				if (direction_index==6) {
					// restart with direction 0.
					direction_index = 0;	
					// use all so far produced colors as new seed colors.
					seed_count = colors.size();
					// take half step distance.
					distance = distance / 2;
				}
				else {
					// go to next direction.
					direction_index = direction_index + 1;
				}					
			
			}
		
		}
		
		return result;
	
	}
	
	*/

}
