// ***********************************************************************
// VISUAL MARRIAGE Applet
//
// Visualizes the algorithm for finding a man-optimal stable marriage.
// The simulated algorithm is the one of Gale and Shapley.
//
// by Rolf Bruderer, Oct 2003
// ***********************************************************************

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.table.*;
import java.util.*;

public class PersonTable
	// Table for visualizing a PersonTableModel.

extends 
	JScrollPane
	
implements
	MouseListener,
	MouseMotionListener,
	ActionListener
{

   	private JTable table;
	// the table visualizing the preference table model.	
	
	private PersonTableModel model;
	// the person table model that is visualized by the table.
	
	private PreferenceListRenderer preference_renderer;
	// special renderer for PreferenceLists in table.
	
	private PersonSymbolRenderer symbol_renderer;
	// special renderer. for rendering persons in table with their symbol.
	
	private MainWindow main_window;
	// the parent main_window this table belong to.
	
	private JPopupMenu popup_menu;
	// the popup menu for this table.	
	
	private JMenuItem popup_mix_item, popup_unmix_item, popup_invert_item, popup_mix_all_item, popup_unmix_all_item, popup_invert_all_item;
	// menu items of popup_menu.
	
	public PersonTable (MainWindow mainwin) 
	// initialize empty preference table with main_window 'mainwin' it belongs to.
	{
	
		// initialize main_window to 'mainwin'.
		main_window = mainwin;
	
		// initialize model to empty.
		model = new PersonTableModel();
		
		// build table.
		buildTable();
	}
	
	
	public PersonTable (PersonTableModel m, MainWindow mainwin) 
	// initialize preference table with model `m' and main window 'mainwin' it belongs to.
	{
	
		// set main_window to mainwin.
		main_window = mainwin;

		// set model to m.
		model = m;
				
		// build table.
		buildTable();					
	}
	

	private JMenuItem getAMenuItem(String text) {
		JMenuItem item = new JMenuItem(text);
		item.setActionCommand(text);
		item.addActionListener(this);
		return item;
	}
	
	
	private void buildTable ()
	// build table.
	{		
	
		//create popup_menu.
		popup_menu = new JPopupMenu();
		popup_menu.add(getAMenuItem("mix all"));
		popup_menu.add(getAMenuItem("unmix all"));
		popup_menu.add(getAMenuItem("invert all"));		
		popup_menu.addSeparator();
		popup_menu.add(getAMenuItem("mix row"));
		popup_menu.add(getAMenuItem("unmix row"));
		popup_menu.add(getAMenuItem("invert row"));	
	
		
		// create table.	
		table = new JTable (model);
		table.setRowHeight(18);
		preference_renderer = new PreferenceListRenderer();
		symbol_renderer = new PersonSymbolRenderer();
		table.setDefaultRenderer (PreferenceList.class, preference_renderer);
		table.setDefaultRenderer (Person.class, symbol_renderer);
		table.setAutoResizeMode (JTable.AUTO_RESIZE_OFF);
		table.setBackground (Color.black);
		table.setForeground (Color.white);
		table.setRowSelectionAllowed(false);
		((JLabel) table.getTableHeader ().getDefaultRenderer ()).setHorizontalAlignment(JLabel.LEFT);
		setViewportView (table);
		getViewport().setBackground(Color.black);
		
		table.addMouseListener(this);
		table.addMouseMotionListener(this);				
	}
	
	public void setModel (PersonTableModel m)
	// set table model to `m'.
	{
		model = m;
		table.setModel (m);
	}
	
	public JTable getTable() 
	// return the jtable-object of this person table.
	{
		return table;
	}
	
	public void showSymbolsOnly() 
	// configure the preference_renderer to show preferences as symbols.
	{
		preference_renderer.showSymbolsOnly();		
		table.repaint ();
	}
	
	public void showNamesOnly() 
	// configure the preference_renderer to show preferences as symbols.
	{
		preference_renderer.showNamesOnly();
		table.repaint ();
	}
	
	public void showSymbolsAndNames() 
	// configure the preference_renderer to show preferences as symbols and names.
	{
		preference_renderer.showSymbolsAndNames();
		table.repaint ();
	}

	
	public void adjustPreferenceColumnWidth ()
	// choose width of preference-column according to number of preferences.
	{

		// is there at least one row? 
		// otherwise nothing has to be done.
		if (model.getRowCount () > 0) {
		
			PreferenceList preferences = (PreferenceList) model.getValueAt (0, 2);
	
			// determine number of preferences.
			int count_preferences = preferences.size ();
			
			// determine available place on screen for this column.
			int available_width = getWidth() - 21 - table.getColumn (model.getColumnIdentifier (0)).getWidth () - table.getColumn (model.getColumnIdentifier (1)).getWidth ();
			available_width = available_width - (available_width % count_preferences);
			
			// set preferences column width.
			TableColumn preferences_col = table.getColumn (model.getColumnIdentifier (2));
			if (count_preferences * 20 + 4 < available_width) {
				preferences_col.setMinWidth (count_preferences * 20 + 4);
				preferences_col.setPreferredWidth (available_width + 3);
			}
			else {
				preferences_col.setMinWidth (available_width + 3);
				preferences_col.setPreferredWidth (count_preferences * 20 + 4);	
			}
			

		}
		
	}
	
	public void paint(Graphics g) 
	// paint table with antialiasing on.
	{
	
		Graphics2D g2d = (Graphics2D)g;		
		g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
		super.paint(g2d);		
		
	}	
	
	private int popup_row_index;
	// the index of the row in which the popup_menu has been triggered.
	
    private void maybeShowPopup(MouseEvent e) 
	// show popup menu, if user wnated to.
	{
        if (e.isPopupTrigger()) {
				
			// determine 'popup_row_index' and select row;
			popup_row_index = table.rowAtPoint(e.getPoint());

			// if mouse cursor is in a row, then select the row.
			if (popup_row_index != -1) {
				table.setCellSelectionEnabled(true);
				table.changeSelection(popup_row_index, 2, false, false);				
			}
			
			// show 'popup_menu'		
            popup_menu.show(e.getComponent(), e.getX(), e.getY());			
        }
    }	

	private int rowBeeingDragged;
	// The number of the row that is beeing dragged
	// (for changing the preference list in).
	
	private int preferenceIndexBeeingDragged;
	// Index of the person in the preference list that is beeing dragged.
	
	private Color colorOfPrefBeeingDragged;
	// old highlight color before the preference has been dragging. 
	// (to be restored after dragging)
	
	public void mousePressed(MouseEvent e) 
	// For drag and drop of preferences, 
	// gets invoked when mouse is beeing pressed.
	// Specify the person that has to be dragged
	// in the actually selected preference list.
	{
	
		maybeShowPopup(e);

		// determine row and set default (invalid) preference index.
		preferenceIndexBeeingDragged = -1;
		rowBeeingDragged = table.rowAtPoint(e.getPoint());	
		
		// determine number of preferences.
		PreferenceList preferences = (PreferenceList) model.getValueAt (0, 2);		
		int count_preferences = preferences.size ();
		
		if ((rowBeeingDragged != -1) && (table.columnAtPoint(e.getPoint()) != -1)) {
		
			if (table.getColumnClass(table.columnAtPoint(e.getPoint())).getName() == "PreferenceList") {		
			
				// determine the selected preference index.
				Rectangle rect = table.getCellRect(rowBeeingDragged, table.columnAtPoint(e.getPoint()), false);
				preferenceIndexBeeingDragged = (int)(e.getX() - rect.getX()) / preference_renderer.fieldWidth;
			}
			
		}		
		
	    // only start dragging if index is valid.
		if ((preferenceIndexBeeingDragged >= 0) && (preferenceIndexBeeingDragged < preferences.size())) {		
			preferences = (PreferenceList) model.getValueAt (rowBeeingDragged, 2);
			Person p = (Person)preferences.get(preferenceIndexBeeingDragged);
			colorOfPrefBeeingDragged = preferences.getBackgroundColor(p);
			preferences.setBackgroundColor(p, table.getSelectionBackground());
			model.fireTableCellUpdated(rowBeeingDragged, 2);
		}
		else {			
			preferenceIndexBeeingDragged = -1;
			rowBeeingDragged = -1;
		}

    }

    public void mouseReleased(MouseEvent e) 
	// For drag and drop of preferences, 
	// gets invoked when mouse is beeing released.
	// Just clear the index of the preference
	// that has been dragged.
	{
	
		maybeShowPopup(e);
	
		if (rowBeeingDragged != -1) {
			// reset drag-highlight of dragged preference.
			PreferenceList preferences = (PreferenceList) model.getValueAt (rowBeeingDragged, 2);
			Person p = (Person)preferences.get(preferenceIndexBeeingDragged);
			preferences.setBackgroundColor(p, colorOfPrefBeeingDragged);		
			model.fireTableCellUpdated(rowBeeingDragged, 2);
		}		

	    // stop dragging
        rowBeeingDragged = -1;
	    preferenceIndexBeeingDragged = -1;
		
    }

 	public void mouseEntered(MouseEvent e) 
	// Part of MouseListener, not implemented here.
	{
    }
	
	public void mouseExited(MouseEvent e) 
	// Part of MouseListener, not implemented here.
	{
    }

    public void mouseClicked(MouseEvent e) 
	// Part of MouseListener, not implemented here.
	{
		maybeShowPopup(e);    
    }
	
    public void mouseMoved(MouseEvent e) 
	// Part of MouseMotionListener, not implemented here.
	{
    }

    public void mouseDragged(MouseEvent e) 
	// For drag and drop of preferences, 
	// gets invoked when mouse is beeing dragged.
	// Move the dragged person to the correct position
	// in the actually selected preference list.
	{

		if ((rowBeeingDragged != -1) && (table.columnAtPoint(e.getPoint()) != -1)) {

			// Get preference list to be edited.
			PreferenceList preferences = (PreferenceList) model.getValueAt (rowBeeingDragged, 2);
		
			// determine new position for dragged preference.
			if (table.getColumnClass(table.columnAtPoint(e.getPoint())).getName() == "PreferenceList") {
			
			    // determine the selected preference index.
			    Rectangle rect = table.getCellRect(rowBeeingDragged, table.columnAtPoint(e.getPoint()), false);
			    int newIndex = (int)(e.getX() - rect.getX()) / preference_renderer.fieldWidth;

				// only do dragging, if new index is valid and diffrent than old index.		    
   			    if ((newIndex >= 0) && (newIndex < preferences.size()) && (newIndex != preferenceIndexBeeingDragged)) {
				
					// first check that world is not running,
					// otherwise stop and reset the world.
					if (!main_window.is_reseted) {

						main_window.reset();
						
						// also rehighlight the color to dragging, after resetting.
						Person p = (Person)preferences.get(preferenceIndexBeeingDragged);
						colorOfPrefBeeingDragged = preferences.getBackgroundColor(p);
						preferences.setBackgroundColor(p, table.getSelectionBackground());
					}

				
					// move from old preference index to new one.					
					preferences.drag(preferenceIndexBeeingDragged, newIndex);
					table.setRowSelectionInterval(rowBeeingDragged, rowBeeingDragged);
					table.setColumnSelectionInterval(2, 2);
			        model.fireTableCellUpdated(rowBeeingDragged, 2);
			    	preferenceIndexBeeingDragged = newIndex;					
					
				}

			}
		
		}
		
		
    }	
	
	public void actionPerformed(ActionEvent e) {
	
		if (e.getActionCommand().equals("mix row")) {
			if (popup_row_index != -1) {
				if (!main_window.is_reseted) {
					main_window.reset();
				}
				model.elementAt(popup_row_index).randomizePreferences();
				model.fireTableRowsUpdated(popup_row_index, popup_row_index);
			}		
	    }		
		if (e.getActionCommand().equals("mix all")) {
			if (!main_window.is_reseted) {
				main_window.reset();
			}
			for (int i = 0; i< model.getRowCount(); i++) {
				model.elementAt(i).randomizePreferences();
			}
			model.fireTableDataChanged();			
	    }
		
	    if (e.getActionCommand().equals("unmix row")) {
			if (popup_row_index != -1) {
				if (!main_window.is_reseted) {
					main_window.reset();
				}
				model.elementAt(popup_row_index).unrandomizePreferences();
				model.fireTableRowsUpdated(popup_row_index, popup_row_index);
			}		
	    }		
		if (e.getActionCommand().equals("unmix all")) {
			if (!main_window.is_reseted) {
				main_window.reset();
			}
			for (int i = 0; i< model.getRowCount(); i++) {
				model.elementAt(i).unrandomizePreferences();
			}
			model.fireTableDataChanged();			
	    }
		
	    if (e.getActionCommand().equals("invert row")) {
			if (popup_row_index != -1) {
				if (!main_window.is_reseted) {
					main_window.reset();
				}
				model.elementAt(popup_row_index).invertPreferences();
				model.fireTableRowsUpdated(popup_row_index, popup_row_index);
			}		
	    }		
		if (e.getActionCommand().equals("invert all")) {
			if (!main_window.is_reseted) {
				main_window.reset();
			}	
			for (int i = 0; i< model.getRowCount(); i++) {
				model.elementAt(i).invertPreferences();
			}
			model.fireTableDataChanged();			
	    }
		
		
		
   	}

}
