// ***********************************************************************
// VISUAL MARRIAGE Applet
//
// Visualizes the algorithm for finding a man-optimal stable marriage.
// The simulated algorithm is the one of Gale and Shapley.
//
// by Rolf Bruderer, Oct 2003
// ***********************************************************************

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.table.*;
import java.util.*;


public class PreferenceListRenderer
	// TableCellRenderer for PreferenceLists.
	
extends JPanel

implements 

	TableCellRenderer
	
{

	DefaultTableCellRenderer default_renderer;
	// a default table cell renderer.
		
	private boolean show_prefs_as_symbols;
	// does the renderer visualize preferences by symbols?
	
	private boolean show_prefs_as_names;
	// does the renderer visualize preferences by names?
	
    public PreferenceListRenderer () 
	// initialize.
	{
		setOpaque(true);
		default_renderer = new DefaultTableCellRenderer();
		setLayout (new GridLayout (1, 0));
		show_prefs_as_symbols = true;			
	}
	
	public int fieldWidth = 1;
	// the last used field width when rendering a preference list.
	
	public void paint(Graphics g) 
	// paint preference list with antialiasing on.
	{	

		Graphics2D g2d = (Graphics2D)g;		
		g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
		super.paint(g2d);	
		
		fieldWidth = getComponent(0).getWidth();
		
	}
	
	public void showSymbolsOnly() 
	// configure this renderer to show preferences as symbols.
	{
		show_prefs_as_symbols = true;
		show_prefs_as_names = false;
	}
	
	public void showNamesOnly() 
	// configure this renderer to show preferences as names.
	{
		show_prefs_as_names = true;
		show_prefs_as_symbols = false;
	}
	
	public void showSymbolsAndNames() 
	// configure this renderer to show preferences as symbols and names.
	{
		show_prefs_as_names = true;
		show_prefs_as_symbols = true;
	}


	public Component getTableCellRendererComponent (JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column)
	// renders a PreferenceList in a Table.
	{	
		

		// empty panel.
		removeAll(); 
		
		// take properties from default panel.
		JLabel c = (JLabel) default_renderer.getTableCellRendererComponent (table, value, isSelected, hasFocus, row, column);
		setBackground (c.getBackground ());		
		setBorder (c.getBorder());
		
		// get preferences to visualize.
		PreferenceList preferences = (PreferenceList) value;
		
		// visualize preferences either as names or as symbols.
		Enumeration enum_preferences = preferences.elements();
		while (enum_preferences.hasMoreElements ()) {
			Person p = (Person) enum_preferences.nextElement ();

			// get component to visualize preference field (either as symbol or as name).
			JComponent component;
			if (show_prefs_as_symbols) {
				if (show_prefs_as_names) {
					component = p.getSymbolWithNameComponent ();
				}
				else {
					component = p.getSymbolComponent ();
				}
			}
			else {
				component = p.getNameComponent ();
			}
			
			// set border for preference field.
			Color highlightColor = preferences.getHighlightColor(p);
			if (highlightColor == null) {
				if (enum_preferences.hasMoreElements ()) {
					// use default simple border (no highlight).
					component.setBorder (BorderFactory.createMatteBorder (0, 0, 0, 1, table.getGridColor ()));
				}
				else {
					//if last one, just no border at all!
					component.setBorder (BorderFactory.createMatteBorder (0, 0, 0, 0, table.getGridColor ()));
				}
			}
			else {
				component.setBorder (BorderFactory.createMatteBorder (1, 1, 1, 1, highlightColor));
			}
			
			// Set background for preference field.
			Color backColor = preferences.getBackgroundColor(p);
			if (backColor == null) {
				// use black background (if no background color set).
				backColor = Color.black;
			}
			if (backColor == Color.black) {
				// use white foreground if background is black.
				component.setForeground(Color.white);
			}
			else {
				// otherwise use black foreground.
				component.setForeground(Color.black);
			}
			component.setBackground(backColor);
			add (component);					
		}
		
		// return the rendered cell.
       	return this;	    
		
	}
		
}
