// ***********************************************************************
// VISUAL MARRIAGE Applet
//
// Visualizes the algorithm for finding a man-optimal stable marriage.
// The simulated algorithm is the one of Gale and Shapley.
//
// by Rolf Bruderer, Oct 2003
// ***********************************************************************

import java.util.Random;
import java.awt.*;
import java.awt.font.*;
import java.awt.geom.*;
import javax.swing.*;
import java.util.*;

public class Woman
	// Women that wait for mens in a world.
extends Person 
{

	public Woman ()
		// Create and initialize
	{
		rnd = new Random();
		highlighted = false;		
		pendingAction = false;
		countAdmirer = 0;  // currently not locked.
	}


	public Woman (int xp, int yp)
		// Create and initialize
		// with x and y position `xp' and `yp'.
	{
		x = xp;
		y = yp;
		rnd = new Random();
		highlighted = false;
		pendingAction = false;
		countAdmirer = 0;  // currently not locked.
	}
	
	static Random rnd;
		// just some randomizer.
	
	boolean highlighted;
		// Is this woman involved in the current pending proposal?
		// If yes she has to be painted highlighted and also visualized specialy in the women table.
	
	int frameHighlighted; 
		// frame counter for animation of highlighted state.
	
	int countAdmirer;
		// number of men that are involved in the pending proposal phase and still are admiring this woman.
		// is the number of men that has to call 'endProposal' to unlock 'pendingAction'.
	
	Person admirer1, admirer2;
		// The two men involved in the proposal (the one doing the proposal and the old partner if she was married before).
	
	public void reset()
	// reset woman.
	{
		super.reset();
		highlighted = false;
		pendingAction = false;
		countAdmirer = 0;  // currently not locked.		
	}
		
	public void step()
	// let the woman do the next step of its movement.
	// (just does nothing)
	{
		// does no movement, just waits for mens to make her a proposal.
	}
	
	public void paint (Graphics g)
	// Paint this person in a world
	// with Graphics `g' at its position.
	{
		int offset;
	
		// paint highlight if necessary.
		if (highlighted) {

			frameHighlighted = (frameHighlighted + 1) % (int)(size);
			
			offset = (int)(1.75*size) - (int)(frameHighlighted * 0.85);
			
			double colorfact = 0.33 + (frameHighlighted / size)*0.66;
			
			g.setColor(new Color ((int)(255.0 * colorfact), (int)(170.0 * colorfact), 0));
			g.fillOval((int)(x - offset), (int)(y - offset), (int)(size + 2*offset), (int)(size + 2*offset));
						
			g.setColor(Color.black);
			int width = (int)(0.38 * size);
			g.fillRect((int)(x - offset), (int)(y + size/2 - width/2), (int)(size + 2*offset), width);
			g.fillRect((int)(x + size/2 - width/2), (int)(y - offset), width, (int)(size + 2*offset));
			offset = offset - width;			
			g.fillOval((int)(x - offset), (int)(y - offset), (int)(size + 2*offset), (int)(size + 2*offset));


			
		}

		// paint the person as filled ring, with black center.
		offset = (int)(size / 2);		
		g.setColor (color);				
		/*
		if (!highlighted && pendingAction) {
			// choose 50% darker color.
			g.setColor (new Color(color.getRed() / 2, color.getGreen() / 2, color.getBlue() / 2));
		}
		*/
		g.fillOval ((int)(x - offset), (int)(y - offset), (int)(size + 2 * offset), (int)(size + 2 * offset));		
		g.setColor (Color.black);
		g.fillOval ((int)x, (int)y, (int)size, (int)size);
		
		// paint the women's admirers on top.
		if (admirer2 != null) {
			admirer2.paint(g);			
		}
		if (admirer1 != null) {   // the one that did the proposal should be on top.
			admirer1.paint(g);			
		}
		
		//prepare Graphics and Font for drawing names.
 	    Graphics2D g2 = (Graphics2D)g;
		Font name_font = g2.getFont().deriveFont(Font.PLAIN);

		// Show the actor's names if highlighted.
		if (highlighted) {
			if (is_acting_names_visible) {
			
		 	   g2.setColor (color);
			   TextLayout tl = new TextLayout(name, name_font, g2.getFontRenderContext());
	           Rectangle2D bounds = tl.getBounds();
			   double yNameWoman = y + 2*size + bounds.getHeight();
	           tl.draw(g2, (float)(x + size/2 - bounds.getWidth()/2), (float)(yNameWoman));

			   if (admirer1 != null) {
				   g.setColor (admirer1.color);
		           tl = new TextLayout(admirer1.name, name_font, g2.getFontRenderContext());
		           bounds = tl.getBounds();
	    	       tl.draw(g2, (float)(x + size/2 - bounds.getWidth()/2), (float)(y - size));
			   }

			   if (admirer2 != null) {
				   g.setColor (admirer2.color);
		           tl = new TextLayout(admirer2.name, name_font, g2.getFontRenderContext());
		           bounds = tl.getBounds();
	    	       tl.draw(g2, (float)(x + size/2 - bounds.getWidth()/2), (float)(yNameWoman + 3 + bounds.getHeight()));
			   }
			}		
		}
		else {
			// if not highlighted show married names, if the user wants to.
			if (is_married_names_visible && (partner != null)) {
			
			   g2.setColor (color);
			   TextLayout tl = new TextLayout(name, name_font, g2.getFontRenderContext());
	           Rectangle2D bounds = tl.getBounds();
			   double yNameWoman = y + 2*size + bounds.getHeight();
	           tl.draw(g2, (float)(x + size/2 - bounds.getWidth()/2), (float)(yNameWoman));

			   g2.setColor (partner.color);
	           tl = new TextLayout(partner.name, name_font, g2.getFontRenderContext());
	           bounds = tl.getBounds();
    	       tl.draw(g2, (float)(x + size/2 - bounds.getWidth()/2), (float)(yNameWoman + 3 + bounds.getHeight()));
			   		
			}
		}
			
		
		 
	}
	
	
	public void paintIcon(Component c,
                      Graphics g,
                      int x,
                      int y)
	// Paint the icon symbol of this person
	// with graphics `g' on to component `c'
	// at location `x', `y'.
	{
		Graphics2D g2d = (Graphics2D)g;		
		g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);

		// paint the person as filled ring, with black center.
		g.setColor (color);
		g.fillOval ((int)x, (int)y, 12, 12);	
		g.setColor (Color.black);
		g.fillOval ((int)x + 3, (int)y + 3, 6, 6);
	}
	
	public void startProposal(Man sender)
	// prepare woman to receive a proposal:
	// highlight her and visualize the proposal in the women table.
	{
		highlighted = true;
		frameHighlighted = 0;
		pendingAction = true;
		countAdmirer = 1;  // per default there is only one man that has to end the proposal phase.
		admirer1 = sender;		
		admirer2 = partner;
		if (partner != null) {
			preferences.setBackgroundColor(partner, Color.orange);
		}
		preferences.setBackgroundColor(sender, Color.orange);	
		ownerTable.fireTableRowsUpdated(ownerTableRow, ownerTableRow);
		
		// log this action to history_window.
		history_window.addProposalMessage(sender, this);
	}
	
	public boolean acceptProposal (Man sender) 
	// return true if this woman accepts the proposal and marries the sender.
	// otherwise return false.
	// set countAdmirer to 2 if she divorces (because then both men have to end the proposal phase).
	{

		// not yet married?
		if (!married) {
			// marry him.			
			married = true;
			partner = sender;
			preferences.setHighlightColor(partner, Color.yellow);
			preferences.setBackgroundColor(partner, null);			
			ownerTable.fireTableRowsUpdated(ownerTableRow, ownerTableRow);
						
			// log this action to history_window.
			history_window.addFreeWomanMessage(this);
			history_window.addMarriageMessage(this, partner);
			
			return true;
		}
		
		// log to history window, that woman is already married.
		history_window.addMarriedWomanMessage(this, partner);
				
		
		// go through preferences 
		// and look if the sender is more preferred 
		// than the actual partner.
		Enumeration enum = preferences.elements ();
		Person p = (Person) enum.nextElement ();
		while ((p != partner) && (p != sender)) {
			p = (Person) enum.nextElement ();
		}
		
		// can they marry??
		if (p == sender) {
		
			// log to history window, that she loves 'sender' more.
			history_window.addLoveMessage("but", sender, partner);
					
			// marry him.
			((Man)partner).divorce ();
			countAdmirer = 2;
			preferences.setHighlightColor(partner, Color.red);
			preferences.setBackgroundColor(partner, null);
			partner = sender;
			preferences.setHighlightColor(partner, Color.yellow);
			preferences.setBackgroundColor(partner, null);					
			history_window.addMarriageMessage(this, partner);	
			ownerTable.fireTableRowsUpdated(ownerTableRow, ownerTableRow);			
			return true;
		}
		
		// log to history window, that she loves 'partner' more.
		history_window.addLoveMessage("and", partner, sender);
				
		// no they cant, so unhighlight them and return false.
		preferences.setHighlightColor(partner, Color.yellow);
		preferences.setBackgroundColor(partner, null);			
		preferences.setHighlightColor(sender, Color.red);
		preferences.setBackgroundColor(sender, null);						
		history_window.addDenialMessage(partner);
		ownerTable.fireTableRowsUpdated(ownerTableRow, ownerTableRow);				
		return false;
	}
	
	public void endProposal()
	// ends the proposal phase for this woman:
	// decrease countAdmirer and if countAdmirer reaches 0
	// unhighlight her and stop visualizing the proposal in the women table.	
	{
		if (countAdmirer > 0) {
			countAdmirer--;
			if (countAdmirer == 0) {
				highlighted = false;
				pendingAction = false;				
			}
		}
	}
	
	public boolean isActing() {
		return highlighted;   // woman handles the whole acting for herself and the admirers.
	}	
			
}
